package multidimensional.arrays1;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.ComboBox;
import javafx.collections.ObservableList;
import javafx.collections.FXCollections;
import javafx.scene.control.TextArea;
import javafx.scene.control.Button;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

/**
 * A JavaFX application that demonstrates two-dimensional arrays.
 *
 * @author Prof. Carl B. Struck
 * @version Copyright (c) SCCC Spring 2018 - SCCC West (First version - 2002)
 */
public class MultidimensionalArrays1 extends Application
{
    private ComboBox comboBoxStart;
    private ComboBox comboBoxEnd;
    private TextArea textAreaResults;
    private Button buttonSearch;

    // A multidimensional array of mileages between cities
    private static int[][] mileage =
    {
        {  0, 160, 390,  40},
        {160,   0, 240, 200},
        {390, 240,   0, 440},
        { 40, 200, 440,   0}
    };
    
    /**
     * Creates a JavaFX window with two ComboBoxes for input and a TextArea and 
     * a Button with an event handler method that demonstrates the search of
     * a two-dimensional array search of distances between cities.
     * 
     * @param primaryStage a Stage window object 
     */
    @Override
    public void start(Stage primaryStage) 
    {
        ObservableList<String> cities = FXCollections.observableArrayList(
            "New York City", "Albany", "Buffalo", "Brentwood" );
        
        comboBoxStart = new ComboBox(cities);
        comboBoxStart.setPrefWidth(150);
        
        comboBoxEnd = new ComboBox(cities);
        comboBoxEnd.setPrefWidth(150);
        
        HBox hBoxInput = new HBox(10, new Label("Start city:"), comboBoxStart, new Label("  End city:"), comboBoxEnd);
        hBoxInput.setPadding( new Insets(10) );
        hBoxInput.setAlignment(Pos.CENTER);
        
        textAreaResults = new TextArea();
        textAreaResults.setPrefSize(530, 175);
        textAreaResults.setEditable(false);

        buttonSearch = new Button("Distance");
        buttonSearch.setStyle("-fx-text-fill: white; -fx-background-color: midnightblue; -fx-font-weight: bold;");
        buttonSearch.setOnAction( e -> begin(e) );

        VBox vBoxSearch = new VBox(10, hBoxInput, textAreaResults, buttonSearch);
        vBoxSearch.setPadding( new Insets(10) );
        vBoxSearch.setAlignment(Pos.CENTER);

        Scene scene = new Scene(vBoxSearch, 550, 250);

        primaryStage.setTitle("Multidimensional Arrays");
        primaryStage.setScene(scene);
        primaryStage.show();
    }
    /**
     * Gets the indexes for two cities from the ComboBoxes and displays the 
     * distance in miles between those cities.
     * 
     * @param event the ActionEvent parameter
     * 
     * @throws NullPointerException if either city is missing from drop down lists
     */
    public void begin(ActionEvent event) 
    {
        try
        {
            int begin = comboBoxStart.getSelectionModel().getSelectedIndex();
            int end = comboBoxEnd.getSelectionModel().getSelectedIndex();
            
            String beginString = comboBoxStart.getSelectionModel().getSelectedItem().toString();
            String endString = comboBoxEnd.getSelectionModel().getSelectedItem().toString();

            textAreaResults.setText( String.format("Distance between %s and %s is %d miles", 
                    beginString, endString, mileage[begin][end] ) );
        }

        catch (NullPointerException ex)
        {
            textAreaResults.setText("Select both 'Start' and 'End' cities from drop down lists");
        }

        catch (Exception ex)
        {
            textAreaResults.setText( ex.toString() );
        }
    }

    /**
     * @param args the command line String[] array arguments
     */
    public static void main(String[] args) 
    {
        launch(args);
    }
}
