package multidimensional.arrays2;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.ComboBox;
import javafx.collections.ObservableList;
import javafx.collections.FXCollections;
import javafx.scene.control.TextArea;
import javafx.scene.control.Button;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

/**
 * A JavaFX application that demonstrates two-dimensional arrays.
 *
 * @author Prof. Carl B. Struck
 * @version Copyright (c) SCCC Spring 2018 - SCCC West
 */
public class MultidimensionalArrays2 extends Application 
{
    private ComboBox comboBoxStudents;
    private ComboBox comboBoxExams;
    private TextArea textAreaResults;
    private Button buttonSearch;

    // A multidimensional array of student exam scores
    private int[][] scores =
    {
        {100,  90,  80},
        { 92,  94,  96},
        { 60,  70,  82},
        { 99,  90,  94},
        { 90,  40,  82},
        { 91, 100,  86}
    };
    
    /**
     * Creates a JavaFX window with two ComboBoxes for input and a TextArea and 
     * a Button with an event handler method that demonstrates the search of
     * a two-dimensional array search of students and exam scores.
     * 
     * @param primaryStage a Stage window object 
     */
    @Override
    public void start(Stage primaryStage) 
    {
        ObservableList<String> students = FXCollections.observableArrayList(
            "Allen", "Byron", "Cindy", "Deborah", "Frances", "George" );
        
        ObservableList<String> exams = FXCollections.observableArrayList(
            "Exam 1", "Exam 2", "Final Exam" );
        
        comboBoxStudents = new ComboBox(students);
        comboBoxStudents.setPrefWidth(125);
        
        comboBoxExams = new ComboBox(exams);
        comboBoxExams.setPrefWidth(125);
        
        HBox hBoxInput = new HBox(10, new Label("Student:"), comboBoxStudents, new Label("  Exam #:"), comboBoxExams);
        hBoxInput.setPadding( new Insets(10) );
        hBoxInput.setAlignment(Pos.CENTER);
        
        textAreaResults = new TextArea();
        textAreaResults.setPrefSize(480, 175);
        textAreaResults.setEditable(false);

        buttonSearch = new Button("Score");
        buttonSearch.setStyle("-fx-text-fill: white; -fx-background-color: midnightblue; -fx-font-weight: bold;");
        buttonSearch.setOnAction( e -> begin(e) );

        VBox vBoxSearch = new VBox(10, hBoxInput, textAreaResults, buttonSearch);
        vBoxSearch.setPadding( new Insets(10) );
        vBoxSearch.setAlignment(Pos.CENTER);

        Scene scene = new Scene(vBoxSearch, 500, 250);

        primaryStage.setTitle("Multidimensional Arrays");
        primaryStage.setScene(scene);
        primaryStage.show();
    }
    /**
     * Gets the indexes for the student and exam from the ComboBoxes and displays 
     * the exam score for that student.
     * 
     * @param event the ActionEvent parameter
     * 
     * @throws NullPointerException if either city is missing from drop down lists
     */
    public void begin(ActionEvent event) 
    {
        try
        {
            int studentID = comboBoxStudents.getSelectionModel().getSelectedIndex();
            int examID = comboBoxExams.getSelectionModel().getSelectedIndex();
            
            String studentName = comboBoxStudents.getSelectionModel().getSelectedItem().toString();
            String examNumber = comboBoxExams.getSelectionModel().getSelectedItem().toString();

            textAreaResults.setText(String.format("%s's score on %s is %d", 
                    studentName, examNumber, scores[studentID][examID] ) );
        }

        catch (NullPointerException ex)
        {
            textAreaResults.setText("Select both 'Student' and 'Exam' from drop down lists");
        }


        catch (Exception ex)
        {
            textAreaResults.setText( ex.toString() );
        }
    }

    /**
     * @param args the command line String[] array arguments
     */
    public static void main(String[] args) 
    {
        launch(args);
    }
}
