package sort.pkg4;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.TextArea;
import javafx.scene.control.Button;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import java.util.Arrays;

/**
 * A JavaFX application that demonstrates a bubble sort.
 *
 * @author Prof. Carl B. Struck
 * @version Copyright (c) SCCC Spring 2018 - SCCC West (First version - 2001)
 */
public class Sort4 extends Application
{
    private TextArea textAreaResults;
    private Button buttonSearch;

    private int[] values;
    private StringBuilder output;

    /**
     * Creates a JavaFX window with a TextField for input and a TextArea and 
     * a Button with an event handler method that demonstrates a linear search.
     * Starts by instantiating an 'int' array of ten elements.
     * 
     * @param primaryStage a Stage window object 
     */
    @Override
    public void start(Stage primaryStage) 
    {
        textAreaResults = new TextArea();
        textAreaResults.setPrefSize(380, 175);
        textAreaResults.setEditable(false);

        buttonSearch = new Button("Sort");
        buttonSearch.setStyle("-fx-text-fill: white; -fx-background-color: midnightblue; -fx-font-weight: bold;");
        buttonSearch.setOnAction( e -> begin(e) );

        VBox vBoxSort = new VBox(10, textAreaResults, buttonSearch);
        vBoxSort.setPadding( new Insets(10) );
        vBoxSort.setAlignment(Pos.CENTER);

        Scene scene = new Scene(vBoxSort, 400, 250);

        primaryStage.setTitle("Selection Sort");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    /**
     * Instantiates an 'int' array of ten elements and assigns random values 
     * to each element, after which it calls the toString() method of class 
     * Arrays to display the array, sorts the array and then displays the array
     * once more.
     *
     * @param event the ActionEvent parameter
     */
    private void begin(ActionEvent event) 
    {
        initArray();
        
        output = new StringBuilder();

        output.append("Before the sort: ");
        output.append( Arrays.toString(values) );
        output.append("\n\n");

        sort();
        output.append("After the sort: ");
        output.append( Arrays.toString(values) );
            
        textAreaResults.setText( output.toString() );
    }

    /**
     * Instantiates an 'int' array of ten elements and assigns random values 
     * to each element.
     */
    private void initArray()
    {
        values = new int[10];

        for (int ctr = 0; ctr < values.length; ctr++)
        {
            values[ctr] = (int) Math.floor( 1 + Math.random() * 100 );
        }
    }

    /**
     * Sorts the array using a bubble sort.
     */
    public void sort()
    {
        for (int x = 0; x < (values.length - 1); x++)
        {
            for (int y = x + 1; y < values.length; y++)
            {
                if ( values[x] > values[y] )
                {
                    swap(x, y);
                }
            }
        }
    }

    /**
     * Switches the values in two array elements; the first becomes the
     * second and the second becomes the first.
     *
     * @param x the index of the first array element
     * @param y the index of the second array element
     */

    public void swap(int x, int y)
    {
        int temp = values[x];
        values[x] = values[y];
        values[y] = temp;
    }

    /**
     * @param args the command line String[] array arguments
     */
    public static void main(String[] args) 
    {
        launch(args);
    }
}