package sort.pkg5;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.TextArea;
import javafx.scene.control.Button;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import java.util.Arrays;

/**
 * A JavaFX application that demonstrates a merge sort.
 *
 * @author Prof. Carl B. Struck
 * @version Copyright (c) SCCC Spring 2018 - SCCC West (First version - 2011)
 */
public class Sort5 extends Application 
{
    private TextArea textAreaResults;
    private Button buttonSearch;

    private int[] values;
    private StringBuilder output;

    /**
     * Creates a JavaFX window with a TextField for input and a TextArea and 
     * a Button with an event handler method that demonstrates a linear search.
     * Starts by instantiating an 'int' array of ten elements.
     * 
     * @param primaryStage a Stage window object 
     */
    @Override
    public void start(Stage primaryStage) 
    {
        textAreaResults = new TextArea();
        textAreaResults.setPrefSize(380, 175);
        textAreaResults.setEditable(false);

        buttonSearch = new Button("Sort");
        buttonSearch.setStyle("-fx-text-fill: white; -fx-background-color: midnightblue; -fx-font-weight: bold;");
        buttonSearch.setOnAction( e -> begin(e) );

        VBox vBoxSort = new VBox(10, textAreaResults, buttonSearch);
        vBoxSort.setPadding( new Insets(10) );
        vBoxSort.setAlignment(Pos.CENTER);

        Scene scene = new Scene(vBoxSort, 400, 250);

        primaryStage.setTitle("Selection Sort");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    /**
     * Instantiates an 'int' array of ten elements and assigns random values 
     * to each element, after which it calls the toString() method of class 
     * Arrays to display the array, sorts the array and then displays the array
     * once more.
     *
     * @param event the ActionEvent parameter
     */
    private void begin(ActionEvent event) 
    {
        initArray();
        
        output = new StringBuilder();

        output.append("Before the sort: ");
        output.append( Arrays.toString(values) );
        output.append("\n\n");

        sort(0, values.length - 1);
        output.append("After the sort: ");
        output.append( Arrays.toString(values) );
            
        textAreaResults.setText( output.toString() );
    }
    


    /**
     * Instantiates an 'int' array of ten elements and assigns random values 
     * to each element.
     */
    private void initArray()
    {
        values = new int[10];

        for (int ctr = 0; ctr < values.length; ctr++)
        {
            values[ctr] = (int) Math.floor( 1 + Math.random() * 100 );
        }
    }

    /**
     * If the sub-array is equal to or more than one element, split
     * the array into two and call this recursive merge sort() method.
     *
     * @param low the beginning element index in the array or sub-array
     * @param high the end element index in the array or sub-array
     */
    private void sort(int low, int high)
    {
        if ( (high - low) >= 1 )
        {
            int middle1 = (low + high) / 2;
            int middle2 = middle1 + 1;

            sort(low, middle1);
            sort(middle2, high);

            merge(low, middle1, middle2, high);
	}
    }

	/**
	 * Merge the two sub-arrays into a single sorted array.
	 *
	 * @param left the beginning element index in the lower array
	 * @param middle1 the end element index in the lower array
	 * @param middle2 the beginning element index in the upper array
	 * @param right the end element index in the upper array
	 */
	private void merge(int left, int middle1, int middle2, int right)
	{
            int leftIndex = left;
            int rightIndex = middle2;
            int combinedIndex = left;
            int[] combined = new int[values.length];

            // Compare left subarray to right subarray and merge until one is complete
            while (leftIndex <= middle1 && rightIndex <= right)
            {
                if ( values[leftIndex] <= values[rightIndex] )
		{
                    combined[combinedIndex++] = values[leftIndex++];
		}
                else
		{
                    combined[combinedIndex++] = values[rightIndex++];
		}
            }

            // Copy remaining elements of the other subarray to end of merged array
            if (leftIndex == middle2)
            {
                while (rightIndex <= right)
                {
                    combined[combinedIndex++] = values[rightIndex++];
		}
            }
            else
            {
                while (leftIndex <= middle1)
		{
                    combined[combinedIndex++] = values[leftIndex++];
		}
            }

            // Copy the temporary merged array elements back to the actual array
            for (int index = left; index <= right; index++)
            {
                values[index] = combined[index];
            }
	}

    /**
     * @param args the command line String[] array arguments
     */
    public static void main(String[] args) 
    {
        launch(args);
    }
}